@@ -18,13 +18,16 @@
 
 from threading import *
 from offlineimap import threadutil
-from offlineimap.threadutil import InstanceLimitedThread
-from offlineimap.ui import UIBase
-import os.path, re
+from offlineimap.ui import getglobalui
+import os.path
+import re
+import sys
+import traceback
 
 class BaseFolder:
     def __init__(self):
         self.uidlock = Lock()
+        self.ui = getglobalui()
         
     def getname(self):
         """Returns name"""
@@ -201,8 +204,8 @@ class BaseFolder:
 
     def syncmessagesto_neguid_msg(self, uid, dest, applyto, register = 1):
         if register:
-            UIBase.getglobalui().registerthread(self.getaccountname())
-        UIBase.getglobalui().copyingmessage(uid, self, applyto)
+            self.ui.registerthread(self.getaccountname())
+        self.ui.copyingmessage(uid, self, applyto)
         successobject = None
         successuid = None
         message = self.getmessage(uid)
@@ -247,7 +250,7 @@ class BaseFolder:
         for uid in uidlist:
             if usethread and usethread.suggeststhreads():
                 usethread.waitforthread()
-                thread = InstanceLimitedThread(\
+                thread = threadutil.InstanceLimitedThread(\
                     usethread.getcopyinstancelimit(),
                     target = self.syncmessagesto_neguid_msg,
                     name = "New msg sync from %s" % self.getvisiblename(),
@@ -266,25 +269,32 @@ class BaseFolder:
         # synced to the status cache.  This is only a problem with
         # self.getmessage().  So, don't call self.getmessage unless
         # really needed.
-        if register:
-            UIBase.getglobalui().registerthread(self.getaccountname())
-        UIBase.getglobalui().copyingmessage(uid, self, applyto)
-        message = ''
-        # If any of the destinations actually stores the message body,
-        # load it up.
-        for object in applyto:
-            if object.storesmessages():
-                message = self.getmessage(uid)
-                break
-        flags = self.getmessageflags(uid)
-        rtime = self.getmessagetime(uid)
-        for object in applyto:
-            newuid = object.savemessage(uid, message, flags, rtime)
-            if newuid > 0 and newuid != uid:
-                # Change the local uid.
-                self.savemessage(newuid, message, flags, rtime)
-                self.deletemessage(uid)
-                uid = newuid
+        try:
+            if register:
+                self.ui.registerthread(self.getaccountname())
+            self.ui.copyingmessage(uid, self, applyto)
+            message = ''
+            # If any of the destinations actually stores the message body,
+            # load it up.
+            
+            for object in applyto:
+                if object.storesmessages():
+                    message = self.getmessage(uid)
+                    break
+            flags = self.getmessageflags(uid)
+            rtime = self.getmessagetime(uid)
+            for object in applyto:
+                newuid = object.savemessage(uid, message, flags, rtime)
+                if newuid > 0 and newuid != uid:
+                    # Change the local uid.
+                    self.savemessage(newuid, message, flags, rtime)
+                    self.deletemessage(uid)
+                    uid = newuid
+        except (KeyboardInterrupt):
+            raise
+        except:
+            self.ui.warn("ERROR attempting to copy message " + str(uid) \
+                 + " for account " + self.getaccountname() + ":" + traceback.format_exc())
         
 
     def syncmessagesto_copy(self, dest, applyto):
@@ -301,7 +311,7 @@ class BaseFolder:
             if not uid in dest_messagelist:
                 if self.suggeststhreads():
                     self.waitforthread()
-                    thread = InstanceLimitedThread(\
+                    thread = threadutil.InstanceLimitedThread(\
                         self.getcopyinstancelimit(),
                         target = self.copymessageto,
                         name = "Copy message %d from %s" % (uid,
@@ -328,7 +338,7 @@ class BaseFolder:
             if not uid in self_messagelist:
                 deletelist.append(uid)
         if len(deletelist):
-            UIBase.getglobalui().deletingmessages(deletelist, applyto)
+            self.ui.deletingmessages(deletelist, applyto)
             for object in applyto:
                 object.deletemessages(deletelist)
 
@@ -369,10 +379,10 @@ class BaseFolder:
 
         for object in applyto:
             for flag in addflaglist.keys():
-                UIBase.getglobalui().addingflags(addflaglist[flag], flag, [object])
+                self.ui.addingflags(addflaglist[flag], flag, [object])
                 object.addmessagesflags(addflaglist[flag], [flag])
             for flag in delflaglist.keys():
-                UIBase.getglobalui().deletingflags(delflaglist[flag], flag, [object])
+                self.ui.deletingflags(delflaglist[flag], flag, [object])
                 object.deletemessagesflags(delflaglist[flag], [flag])
                 
     def syncmessagesto(self, dest, applyto = None):
@@ -384,15 +394,36 @@ class BaseFolder:
         applyto ones should be the ones that "copy" the main action."""
         if applyto == None:
             applyto = [dest]
-            
-        self.syncmessagesto_neguid(dest, applyto)
+
+        try:
+            self.syncmessagesto_neguid(dest, applyto)
+        except (KeyboardInterrupt):
+            raise
+        except:
+            self.ui.warn("ERROR attempting to handle negative uids " \
+                + "for account " + self.getaccountname() + ":" + traceback.format_exc())
+
+        #all threads launched here are in try / except clauses when they copy anyway...
         self.syncmessagesto_copy(dest, applyto)
-        self.syncmessagesto_delete(dest, applyto)
-        
+
+        try:
+            self.syncmessagesto_delete(dest, applyto)
+        except (KeyboardInterrupt):
+            raise
+        except:
+            self.ui.warn("ERROR attempting to delete messages " \
+                + "for account " + self.getaccountname() + ":" + traceback.format_exc())
+
         # Now, the message lists should be identical wrt the uids present.
         # (except for potential negative uids that couldn't be placed
         # anywhere)
 
-        self.syncmessagesto_flags(dest, applyto)
+        try:
+            self.syncmessagesto_flags(dest, applyto)
+        except (KeyboardInterrupt):
+            raise
+        except:
+            self.ui.warn("ERROR attempting to sync flags " \
+                + "for account " + self.getaccountname() + ":" + traceback.format_exc())
         
             
