@@ -16,15 +16,17 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-from Base import BaseFolder
 import imaplib
-from offlineimap import imaputil, imaplibutil
-from offlineimap.ui import UIBase
-from offlineimap.version import versionstr
-import rfc822, time, string, random, binascii, re
+import rfc822
+import string
+import random
+import binascii
+import re
+import time
 from StringIO import StringIO
 from copy import copy
-
+from Base import BaseFolder
+from offlineimap import imaputil, imaplibutil, __version__
 
 class IMAPFolder(BaseFolder):
     def __init__(self, imapserver, name, visiblename, accountname, repository):
@@ -40,6 +42,7 @@ class IMAPFolder(BaseFolder):
         self.repository = repository
         self.randomgenerator = random.Random()
         BaseFolder.__init__(self)
+        #self.ui is set in BaseFolder
 
     def selectro(self, imapobj):
         """Select this folder when we do not need write access.
@@ -84,9 +87,13 @@ class IMAPFolder(BaseFolder):
             # Primes untagged_responses
             imapobj.select(self.getfullname(), readonly = 1, force = 1)
             try:
-                # Some mail servers do not return an EXISTS response if
-                # the folder is empty.
-                maxmsgid = long(imapobj.untagged_responses['EXISTS'][0])
+                # 1. Some mail servers do not return an EXISTS response
+                # if the folder is empty.  2. ZIMBRA servers can return
+                # multiple EXISTS replies in the form 500, 1000, 1500,
+                # 1623 so check for potentially multiple replies.
+                maxmsgid = 0
+                for msgid in imapobj.untagged_responses['EXISTS']:
+                    maxmsgid = max(long(msgid), maxmsgid)
             except KeyError:
                 return True
 
@@ -116,6 +123,7 @@ class IMAPFolder(BaseFolder):
 
         return False
 
+    # TODO: Make this so that it can define a date that would be the oldest messages etc.
     def cachemessagelist(self):
         imapobj = self.imapserver.acquireconnection()
         self.messagelist = {}
@@ -123,20 +131,65 @@ class IMAPFolder(BaseFolder):
         try:
             # Primes untagged_responses
             imapobj.select(self.getfullname(), readonly = 1, force = 1)
-            try:
-                # Some mail servers do not return an EXISTS response if
-                # the folder is empty.
-                maxmsgid = long(imapobj.untagged_responses['EXISTS'][0])
-            except KeyError:
-                return
-            if maxmsgid < 1:
-                # No messages; return
-                return
 
+            maxage = self.config.getdefaultint("Account " + self.accountname, "maxage", -1)
+            maxsize = self.config.getdefaultint("Account " + self.accountname, "maxsize", -1)
+
+            if (maxage != -1) | (maxsize != -1):
+                try:
+                    search_condition = "(";
+
+                    if(maxage != -1):
+                        #find out what the oldest message is that we should look at
+                        oldest_time_struct = time.gmtime(time.time() - (60*60*24*maxage))
+
+                        #format this manually - otherwise locales could cause problems
+                        monthnames_standard = ["Jan", "Feb", "Mar", "Apr", "May", \
+                            "June", "July", "Aug", "Sep", "Oct", "Nov", "Dec"]
+
+                        our_monthname = monthnames_standard[oldest_time_struct[1]-1]
+                        daystr = "%(day)02d" % {'day' : oldest_time_struct[2]}
+                        date_search_str = "SINCE " + daystr + "-" + our_monthname \
+                            + "-" + str(oldest_time_struct[0])
+
+                        search_condition += date_search_str
+
+                    if(maxsize != -1):
+                        if(maxage != 1): #There are two conditions - add a space
+                            search_condition += " "
+
+                        search_condition += "SMALLER " + self.config.getdefault("Account " + self.accountname, "maxsize", -1)
+
+                    search_condition += ")"
+                    searchresult = imapobj.search(None, search_condition)
+
+                    #result would come back seperated by space - to change into a fetch
+                    #statement we need to change space to comma
+                    messagesToFetch = searchresult[1][0].replace(" ", ",")
+                except KeyError:
+                    return
+                if len(messagesToFetch) < 1:
+                    # No messages; return
+                    return
+            else:
+                try:
+                    # 1. Some mail servers do not return an EXISTS response
+                    # if the folder is empty.  2. ZIMBRA servers can return
+                    # multiple EXISTS replies in the form 500, 1000, 1500,
+                    # 1623 so check for potentially multiple replies.
+                    maxmsgid = 0
+                    for msgid in imapobj.untagged_responses['EXISTS']:
+                        maxmsgid = max(long(msgid), maxmsgid)
+                    messagesToFetch = '1:%d' % maxmsgid;
+                except KeyError:
+                    return
+                if maxmsgid < 1:
+                    #no messages; return
+                    return
             # Now, get the flags and UIDs for these.
             # We could conceivably get rid of maxmsgid and just say
             # '1:*' here.
-            response = imapobj.fetch('1:%d' % maxmsgid, '(FLAGS UID)')[1]
+            response = imapobj.fetch(messagesToFetch, '(FLAGS UID)')[1]
         finally:
             self.imapserver.releaseconnection(imapobj)
         for messagestr in response:
@@ -144,7 +197,7 @@ class IMAPFolder(BaseFolder):
             messagestr = string.split(messagestr, maxsplit = 1)[1]
             options = imaputil.flags2hash(messagestr)
             if not options.has_key('UID'):
-                UIBase.getglobalui().warn('No UID in message with options %s' %\
+                self.ui.warn('No UID in message with options %s' %\
                                           str(options),
                                           minor = 1)
             else:
@@ -157,12 +210,11 @@ class IMAPFolder(BaseFolder):
         return self.messagelist
 
     def getmessage(self, uid):
-        ui = UIBase.getglobalui()
         imapobj = self.imapserver.acquireconnection()
         try:
             imapobj.select(self.getfullname(), readonly = 1)
             initialresult = imapobj.uid('fetch', '%d' % uid, '(BODY.PEEK[])')
-            ui.debug('imap', 'Returned object from fetching %d: %s' % \
+            self.ui.debug('imap', 'Returned object from fetching %d: %s' % \
                      (uid, str(initialresult)))
             return initialresult[1][0][1].replace("\r\n", "\n")
                 
@@ -176,40 +228,39 @@ class IMAPFolder(BaseFolder):
         return self.messagelist[uid]['flags']
 
     def savemessage_getnewheader(self, content):
-        headername = 'X-OfflineIMAP-%s-' % str(binascii.crc32(content)).replace('-', 'x')
-        headername += binascii.hexlify(self.repository.getname()) + '-'
-        headername += binascii.hexlify(self.getname())
-        headervalue= '%d-' % long(time.time())
+        headername = 'X-OfflineIMAP'
+        headervalue = '%s-' % str(binascii.crc32(content)).replace('-', 'x')
+        headervalue += binascii.hexlify(self.repository.getname()) + '-'
+        headervalue += binascii.hexlify(self.getname())
+        headervalue += '-%d-' % long(time.time())
         headervalue += str(self.randomgenerator.random()).replace('.', '')
-        headervalue += '-v' + versionstr
+        headervalue += '-v' + __version__
         return (headername, headervalue)
 
     def savemessage_addheader(self, content, headername, headervalue):
-        ui = UIBase.getglobalui()
-        ui.debug('imap',
+        self.ui.debug('imap',
                  'savemessage_addheader: called to add %s: %s' % (headername,
                                                                   headervalue))
         insertionpoint = content.find("\r\n")
-        ui.debug('imap', 'savemessage_addheader: insertionpoint = %d' % insertionpoint)
+        self.ui.debug('imap', 'savemessage_addheader: insertionpoint = %d' % insertionpoint)
         leader = content[0:insertionpoint]
-        ui.debug('imap', 'savemessage_addheader: leader = %s' % repr(leader))
+        self.ui.debug('imap', 'savemessage_addheader: leader = %s' % repr(leader))
         if insertionpoint == 0 or insertionpoint == -1:
             newline = ''
             insertionpoint = 0
         else:
             newline = "\r\n"
         newline += "%s: %s" % (headername, headervalue)
-        ui.debug('imap', 'savemessage_addheader: newline = ' + repr(newline))
+        self.ui.debug('imap', 'savemessage_addheader: newline = ' + repr(newline))
         trailer = content[insertionpoint:]
-        ui.debug('imap', 'savemessage_addheader: trailer = ' + repr(trailer))
+        self.ui.debug('imap', 'savemessage_addheader: trailer = ' + repr(trailer))
         return leader + newline + trailer
 
     def savemessage_searchforheader(self, imapobj, headername, headervalue):
         if imapobj.untagged_responses.has_key('APPENDUID'):
             return long(imapobj.untagged_responses['APPENDUID'][-1].split(' ')[1])
 
-        ui = UIBase.getglobalui()
-        ui.debug('imap', 'savemessage_searchforheader called for %s: %s' % \
+        self.ui.debug('imap', 'savemessage_searchforheader called for %s: %s' % \
                  (headername, headervalue))
         # Now find the UID it got.
         headervalue = imapobj._quote(headervalue)
@@ -217,16 +268,16 @@ class IMAPFolder(BaseFolder):
             matchinguids = imapobj.uid('search', 'HEADER', headername, headervalue)[1][0]
         except imapobj.error, err:
             # IMAP server doesn't implement search or had a problem.
-            ui.debug('imap', "savemessage_searchforheader: got IMAP error '%s' while attempting to UID SEARCH for message with header %s" % (err, headername))
+            self.ui.debug('imap', "savemessage_searchforheader: got IMAP error '%s' while attempting to UID SEARCH for message with header %s" % (err, headername))
             return 0
-        ui.debug('imap', 'savemessage_searchforheader got initial matchinguids: ' + repr(matchinguids))
+        self.ui.debug('imap', 'savemessage_searchforheader got initial matchinguids: ' + repr(matchinguids))
 
         if matchinguids == '':
-            ui.debug('imap', "savemessage_searchforheader: UID SEARCH for message with header %s yielded no results" % headername)
+            self.ui.debug('imap', "savemessage_searchforheader: UID SEARCH for message with header %s yielded no results" % headername)
             return 0
 
         matchinguids = matchinguids.split(' ')
-        ui.debug('imap', 'savemessage_searchforheader: matchinguids now ' + \
+        self.ui.debug('imap', 'savemessage_searchforheader: matchinguids now ' + \
                  repr(matchinguids))
         if len(matchinguids) != 1 or matchinguids[0] == None:
             raise ValueError, "While attempting to find UID for message with header %s, got wrong-sized matchinguids of %s" % (headername, str(matchinguids))
@@ -235,13 +286,12 @@ class IMAPFolder(BaseFolder):
 
     def savemessage(self, uid, content, flags, rtime):
         imapobj = self.imapserver.acquireconnection()
-        ui = UIBase.getglobalui()
-        ui.debug('imap', 'savemessage: called')
+        self.ui.debug('imap', 'savemessage: called')
         try:
             try:
                 imapobj.select(self.getfullname()) # Needed for search
             except imapobj.readonly:
-                ui.msgtoreadonly(self, uid, content, flags)
+                self.ui.msgtoreadonly(self, uid, content, flags)
                 # Return indicating message taken, but no UID assigned.
                 # Fudge it.
                 return 0
@@ -279,17 +329,17 @@ class IMAPFolder(BaseFolder):
                 # but some IMAP servers nonetheless choke on 1902.
                 date = imaplib.Time2Internaldate(time.localtime())
 
-            ui.debug('imap', 'savemessage: using date ' + str(date))
+            self.ui.debug('imap', 'savemessage: using date ' + str(date))
             content = re.sub("(?<!\r)\n", "\r\n", content)
-            ui.debug('imap', 'savemessage: initial content is: ' + repr(content))
+            self.ui.debug('imap', 'savemessage: initial content is: ' + repr(content))
 
             (headername, headervalue) = self.savemessage_getnewheader(content)
-            ui.debug('imap', 'savemessage: new headers are: %s: %s' % \
+            self.ui.debug('imap', 'savemessage: new headers are: %s: %s' % \
                      (headername, headervalue))
             content = self.savemessage_addheader(content, headername,
                                                  headervalue)
-            ui.debug('imap', 'savemessage: new content is: ' + repr(content))
-            ui.debug('imap', 'savemessage: new content length is ' + \
+            self.ui.debug('imap', 'savemessage: new content is: ' + repr(content))
+            self.ui.debug('imap', 'savemessage: new content length is ' + \
                      str(len(content)))
 
             assert(imapobj.append(self.getfullname(),
@@ -300,12 +350,12 @@ class IMAPFolder(BaseFolder):
             assert(imapobj.check()[0] == 'OK')
 
             # Keep trying until we get the UID.
-            ui.debug('imap', 'savemessage: first attempt to get new UID')
+            self.ui.debug('imap', 'savemessage: first attempt to get new UID')
             uid = self.savemessage_searchforheader(imapobj, headername,
                                                    headervalue)
             # See docs for savemessage in Base.py for explanation of this and other return values
             if uid <= 0:
-                ui.debug('imap', 'savemessage: first attempt to get new UID failed.  Going to run a NOOP and try again.')
+                self.ui.debug('imap', 'savemessage: first attempt to get new UID failed.  Going to run a NOOP and try again.')
                 assert(imapobj.noop()[0] == 'OK')
                 uid = self.savemessage_searchforheader(imapobj, headername,
                                                        headervalue)
@@ -315,7 +365,7 @@ class IMAPFolder(BaseFolder):
         if uid: # avoid UID FETCH 0 crash happening later on
             self.messagelist[uid] = {'uid': uid, 'flags': flags}
 
-        ui.debug('imap', 'savemessage: returning %d' % uid)
+        self.ui.debug('imap', 'savemessage: returning %d' % uid)
         return uid
 
     def savemessageflags(self, uid, flags):
@@ -324,7 +374,7 @@ class IMAPFolder(BaseFolder):
             try:
                 imapobj.select(self.getfullname())
             except imapobj.readonly:
-                UIBase.getglobalui().flagstoreadonly(self, [uid], flags)
+                self.ui.flagstoreadonly(self, [uid], flags)
                 return
             result = imapobj.uid('store', '%d' % uid, 'FLAGS',
                                  imaputil.flagsmaildir2imap(flags))
@@ -368,7 +418,7 @@ class IMAPFolder(BaseFolder):
             try:
                 imapobj.select(self.getfullname())
             except imapobj.readonly:
-                UIBase.getglobalui().flagstoreadonly(self, uidlist, flags)
+                self.ui.flagstoreadonly(self, uidlist, flags)
                 return
             r = imapobj.uid('store',
                             imaputil.listjoin(uidlist),
@@ -427,7 +477,7 @@ class IMAPFolder(BaseFolder):
             try:
                 imapobj.select(self.getfullname())
             except imapobj.readonly:
-                UIBase.getglobalui().deletereadonly(self, uidlist)
+                self.ui.deletereadonly(self, uidlist)
                 return
             if self.expunge:
                 assert(imapobj.expunge()[0] == 'OK')
