@@ -18,7 +18,7 @@
 
 import imaplib
 from offlineimap import imaplibutil, imaputil, threadutil
-from offlineimap.ui import UIBase
+from offlineimap.ui import getglobalui
 from threading import *
 import thread, hmac, os, time
 import base64
@@ -59,15 +59,12 @@ class UsefulIMAPMixIn:
     def _mesg(self, s, secs=None):
         imaplibutil.new_mesg(self, s, secs)
 
-class UsefulIMAP4(UsefulIMAPMixIn, imaplib.IMAP4):
-    def open(self, host = '', port = imaplib.IMAP4_PORT):
-        imaplibutil.new_open(self, host, port)
+class UsefulIMAP4(UsefulIMAPMixIn, imaplibutil.WrappedIMAP4):
 
     # This is a hack around Darwin's implementation of realloc() (which
     # Python uses inside the socket code). On Darwin, we split the
     # message into 100k chunks, which should be small enough - smaller
     # might start seriously hurting performance ...
-
     def read(self, size):
         if (system() == 'Darwin') and (size>0) :
             read = 0
@@ -81,12 +78,8 @@ class UsefulIMAP4(UsefulIMAPMixIn, imaplib.IMAP4):
             return imaplib.IMAP4.read (self, size)
 
 class UsefulIMAP4_SSL(UsefulIMAPMixIn, imaplibutil.WrappedIMAP4_SSL):
-    def open(self, host = '', port = imaplib.IMAP4_SSL_PORT):
-        imaplibutil.new_open_ssl(self, host, port)
-
     # This is the same hack as above, to be used in the case of an SSL
     # connexion.
-
     def read(self, size):
         if (system() == 'Darwin') and (size>0) :
             read = 0
@@ -107,7 +100,9 @@ class IMAPServer:
     def __init__(self, config, reposname,
                  username = None, password = None, hostname = None,
                  port = None, ssl = 1, maxconnections = 1, tunnel = None,
-                 reference = '""', sslclientcert = None, sslclientkey = None):
+                 reference = '""', sslclientcert = None, sslclientkey = None,
+                 sslcacertfile= None):
+        self.ui = getglobalui()
         self.reposname = reposname
         self.config = config
         self.username = username
@@ -120,6 +115,7 @@ class IMAPServer:
         self.usessl = ssl
         self.sslclientcert = sslclientcert
         self.sslclientkey = sslclientkey
+        self.sslcacertfile = sslcacertfile
         self.delim = None
         self.root = None
         if port == None:
@@ -145,7 +141,7 @@ class IMAPServer:
         if self.password != None and self.passworderror == None:
             return self.password
 
-        self.password = UIBase.getglobalui().getpass(self.reposname,
+        self.password = self.ui.getpass(self.reposname,
                                                      self.config,
                                                      self.passworderror)
         self.passworderror = None
@@ -172,18 +168,16 @@ class IMAPServer:
         self.semaphore.release()
 
     def md5handler(self, response):
-        ui = UIBase.getglobalui()
         challenge = response.strip()
-        ui.debug('imap', 'md5handler: got challenge %s' % challenge)
+        self.ui.debug('imap', 'md5handler: got challenge %s' % challenge)
 
         passwd = self.getpassword()
         retval = self.username + ' ' + hmac.new(passwd, challenge).hexdigest()
-        ui.debug('imap', 'md5handler: returning %s' % retval)
+        self.ui.debug('imap', 'md5handler: returning %s' % retval)
         return retval
 
     def plainauth(self, imapobj):
-        UIBase.getglobalui().debug('imap',
-                                   'Attempting plain authentication')
+        self.ui.debug('imap', 'Attempting plain authentication')
         imapobj.login(self.username, self.getpassword())
 
     def gssauth(self, response):
@@ -206,8 +200,7 @@ class IMAPServer:
         except kerberos.GSSError, err:
             # Kerberos errored out on us, respond with None to cancel the
             # authentication
-            UIBase.getglobalui().debug('imap',
-                                       '%s: %s' % (err[0][0], err[1][0]))
+            self.ui.debug('imap', '%s: %s' % (err[0][0], err[1][0]))
             return None
 
         if not response:
@@ -254,15 +247,16 @@ class IMAPServer:
             while not success:
                 # Generate a new connection.
                 if self.tunnel:
-                    UIBase.getglobalui().connecting('tunnel', self.tunnel)
+                    self.ui.connecting('tunnel', self.tunnel)
                     imapobj = UsefulIMAP4_Tunnel(self.tunnel)
                     success = 1
                 elif self.usessl:
-                    UIBase.getglobalui().connecting(self.hostname, self.port)
+                    self.ui.connecting(self.hostname, self.port)
                     imapobj = UsefulIMAP4_SSL(self.hostname, self.port,
-                                              self.sslclientkey, self.sslclientcert)
+                                              self.sslclientkey, self.sslclientcert, 
+                                              cacertfile = self.sslcacertfile)
                 else:
-                    UIBase.getglobalui().connecting(self.hostname, self.port)
+                    self.ui.connecting(self.hostname, self.port)
                     imapobj = UsefulIMAP4(self.hostname, self.port)
 
                 imapobj.mustquote = imaplibutil.mustquote
@@ -271,13 +265,13 @@ class IMAPServer:
                     try:
                         # Try GSSAPI and continue if it fails
                         if 'AUTH=GSSAPI' in imapobj.capabilities and have_gss:
-                            UIBase.getglobalui().debug('imap',
+                            self.ui.debug('imap',
                                 'Attempting GSSAPI authentication')
                             try:
                                 imapobj.authenticate('GSSAPI', self.gssauth)
                             except imapobj.error, val:
                                 self.gssapi = False
-                                UIBase.getglobalui().debug('imap',
+                                self.ui.debug('imap',
                                     'GSSAPI Authentication failed')
                             else:
                                 self.gssapi = True
@@ -286,7 +280,7 @@ class IMAPServer:
 
                         if not self.gssapi:
                             if 'AUTH=CRAM-MD5' in imapobj.capabilities:
-                                UIBase.getglobalui().debug('imap',
+                                self.ui.debug('imap',
                                                        'Attempting CRAM-MD5 authentication')
                                 try:
                                     imapobj.authenticate('CRAM-MD5', self.md5handler)
@@ -361,47 +355,46 @@ class IMAPServer:
         until the Event object as passed is true.  This method is expected
         to be invoked in a separate thread, which should be join()'d after
         the event is set."""
-        ui = UIBase.getglobalui()
-        ui.debug('imap', 'keepalive thread started')
+        self.ui.debug('imap', 'keepalive thread started')
         while 1:
-            ui.debug('imap', 'keepalive: top of loop')
+            self.ui.debug('imap', 'keepalive: top of loop')
             time.sleep(timeout)
-            ui.debug('imap', 'keepalive: after wait')
+            self.ui.debug('imap', 'keepalive: after wait')
             if event.isSet():
-                ui.debug('imap', 'keepalive: event is set; exiting')
+                self.ui.debug('imap', 'keepalive: event is set; exiting')
                 return
-            ui.debug('imap', 'keepalive: acquiring connectionlock')
+            self.ui.debug('imap', 'keepalive: acquiring connectionlock')
             self.connectionlock.acquire()
             numconnections = len(self.assignedconnections) + \
                              len(self.availableconnections)
             self.connectionlock.release()
-            ui.debug('imap', 'keepalive: connectionlock released')
+            self.ui.debug('imap', 'keepalive: connectionlock released')
             threads = []
             imapobjs = []
         
             for i in range(numconnections):
-                ui.debug('imap', 'keepalive: processing connection %d of %d' % (i, numconnections))
+                self.ui.debug('imap', 'keepalive: processing connection %d of %d' % (i, numconnections))
                 imapobj = self.acquireconnection()
-                ui.debug('imap', 'keepalive: connection %d acquired' % i)
+                self.ui.debug('imap', 'keepalive: connection %d acquired' % i)
                 imapobjs.append(imapobj)
                 thr = threadutil.ExitNotifyThread(target = imapobj.noop)
                 thr.setDaemon(1)
                 thr.start()
                 threads.append(thr)
-                ui.debug('imap', 'keepalive: thread started')
+                self.ui.debug('imap', 'keepalive: thread started')
 
-            ui.debug('imap', 'keepalive: joining threads')
+            self.ui.debug('imap', 'keepalive: joining threads')
 
             for thr in threads:
                 # Make sure all the commands have completed.
                 thr.join()
 
-            ui.debug('imap', 'keepalive: releasing connections')
+            self.ui.debug('imap', 'keepalive: releasing connections')
 
             for imapobj in imapobjs:
                 self.releaseconnection(imapobj)
 
-            ui.debug('imap', 'keepalive: bottom of loop')
+            self.ui.debug('imap', 'keepalive: bottom of loop')
 
 class ConfigedIMAPServer(IMAPServer):
     """This class is designed for easier initialization given a ConfigParser
@@ -421,6 +414,7 @@ class ConfigedIMAPServer(IMAPServer):
             ssl = self.repos.getssl()
             sslclientcert = self.repos.getsslclientcert()
             sslclientkey = self.repos.getsslclientkey()
+            sslcacertfile = self.repos.getsslcacertfile()
         reference = self.repos.getreference()
         server = None
         password = None
@@ -442,4 +436,5 @@ class ConfigedIMAPServer(IMAPServer):
                                 self.repos.getmaxconnections(),
                                 reference = reference,
                                 sslclientcert = sslclientcert,
-                                sslclientkey = sslclientkey)
+                                sslclientkey = sslclientkey,
+                                sslcacertfile = sslcacertfile)
