@@ -16,10 +16,9 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-from threading import *
+from threading import RLock, currentThread
 from offlineimap.ui.UIBase import UIBase
 import thread
-from offlineimap.threadutil import MultiLock
 
 class BlinkenBase:
     """This is a mix-in class that should be mixed in with either UIBase
@@ -42,6 +41,10 @@ class BlinkenBase:
         s.gettf().setcolor('cyan')
         s.__class__.__bases__[-1].syncingfolder(s, srcrepos, srcfolder, destrepos, destfolder)
 
+    def skippingfolder(s, folder):
+        s.gettf().setcolor('cyan')
+        s.__class__.__bases__[-1].skippingfolder(s, folder)
+
     def loadmessagelist(s, repos, folder):
         s.gettf().setcolor('green')
         s._msg("Scanning folder [%s/%s]" % (s.getnicename(repos),
@@ -71,10 +74,18 @@ class BlinkenBase:
         s.gettf().setcolor('pink')
         s.__class__.__bases__[-1].deletingflags(s, uidlist, flags, destlist)
 
+    def warn(s, msg, minor = 0):
+        if minor:
+            s.gettf().setcolor('pink')
+        else:
+            s.gettf().setcolor('red')
+        s.__class__.__bases__[-1].warn(s, msg, minor)
+
     def init_banner(s):
         s.availablethreadframes = {}
         s.threadframes = {}
-        s.tflock = MultiLock()
+        #tflock protects the s.threadframes manipulation to only happen from 1 thread
+        s.tflock = RLock()
 
     def threadExited(s, thread):
         threadid = thread.threadid
@@ -116,11 +127,15 @@ class BlinkenBase:
             return tf
         finally:
             s.tflock.release()
+
+    def callhook(s, msg):
+        s.gettf().setcolor('white')
+        s.__class__.__bases__[-1].callhook(s, msg)
             
-    def sleep(s, sleepsecs):
+    def sleep(s, sleepsecs, siglistener):
         s.gettf().setcolor('red')
         s.getaccountframe().startsleep(sleepsecs)
-        UIBase.sleep(s, sleepsecs)
+        return UIBase.sleep(s, sleepsecs, siglistener)
 
     def sleeping(s, sleepsecs, remainingsecs):
         if remainingsecs and s.gettf().getcolor() == 'black':
